import { Server } from 'socket.io'
import { TSocket } from '../../../../utilities/types'
import prisma from '../../../../utilities/prisma'
import characterRepository from '../../../../repositories/characterRepository'
import { CharacterGender, CharacterRace } from '@prisma/client'

type Payload = {
  id: number
  name: string
  gender: CharacterGender
  race: CharacterRace
  isSelectable: boolean
  spriteId: string
}

export default class CharacterTypeUpdateEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('gm:characterType:update', this.handleObjectUpdate.bind(this))
  }

  private async handleObjectUpdate(data: Payload, callback: (success: boolean) => void): Promise<void> {
    const character = await characterRepository.getById(this.socket.characterId as number)
    if (!character) return callback(false)

    if (character.role !== 'gm') {
      return callback(false)
    }

    try {
      await prisma.characterType.update({
        where: { id: data.id },
        data: {
          name: data.name,
          gender: data.gender,
          race: data.race,
          isSelectable: data.isSelectable,
          spriteId: data.spriteId
        }
      })

      callback(true)
    } catch (error) {
      console.error(error)
      callback(false)
    }
  }
}
